@@ -6,14 +6,15 @@ use Moo;
 use AE;
 use AnyEvent;
 use AnyEvent::Handle;
-use AnyEvent::Socket qw( tcp_connect );
+use AnyEvent::Socket ();
 use Protocol::WebSocket::Request;
 use Protocol::WebSocket::Handshake::Client;
 use AnyEvent::WebSocket::Connection;
 use PerlX::Maybe qw( maybe provided );
 
 # ABSTRACT: WebSocket client for AnyEvent
-our $VERSION = '0.35_02'; # TRIAL VERSION
+our $VERSION = '0.40_01'; # TRIAL VERSION
+$VERSION = eval $VERSION;
 
 
 has timeout => (
@@ -63,6 +64,11 @@ has http_headers => (
 );
 
 
+has max_payload_size => (
+  is => 'ro',
+);
+
+
 sub connect
 {
   my($self, $uri) = @_;
@@ -82,7 +88,7 @@ sub connect
     return $done;
   }
     
-  tcp_connect $uri->host, $uri->port, sub {
+  AnyEvent::Socket::tcp_connect $uri->host, $uri->port, sub {
     my $fh = shift;
     unless($fh)
     {
@@ -152,9 +158,10 @@ sub connect
         undef $handshake;
         $done->send(
           AnyEvent::WebSocket::Connection->new(
-                  handle      => $hdl,
-                  masked      => 1,
-            maybe subprotocol => $sb,
+                  handle           => $hdl,
+                  masked           => 1,
+            maybe subprotocol      => $sb,
+            maybe max_payload_size => $self->max_payload_size,
           )
         );
         undef $hdl;
@@ -179,7 +186,7 @@ AnyEvent::WebSocket::Client - WebSocket client for AnyEvent
 
 =head1 VERSION
 
-version 0.35_02
+version 0.40_01
 
 =head1 SYNOPSIS
 
@@ -297,6 +304,11 @@ Will generate:
 
 Although, the order cannot be guaranteed when using the hash style.
 
+=head2 max_payload_size
+
+The maximum payload size for received frames.  Currently defaults to whatever
+L<Protocol::WebSocket> defaults to.
+
 =head1 METHODS
 
 =head2 connect
@@ -412,6 +424,8 @@ Toshio Ito
 
 José Joaquín Atria
 
+Kivanc Yazan
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2013 by Graham Ollis.
