@@ -8,10 +8,12 @@ use Scalar::Util ();
 use Encode ();
 use AE;
 use AnyEvent::WebSocket::Message;
+use PerlX::Maybe qw( maybe provided );
 use Carp ();
 
 # ABSTRACT: WebSocket connection for AnyEvent
-our $VERSION = '0.35_02'; # TRIAL VERSION
+our $VERSION = '0.40_01'; # TRIAL VERSION
+$VERSION = eval $VERSION;
 
 
 has handle => (
@@ -30,7 +32,27 @@ has subprotocol => (
   is => 'ro',
 );
 
-foreach my $type (qw( each_message next_message finish ))
+
+has max_payload_size => (
+  is => 'ro',
+);
+
+
+has close_code => (
+  is => 'rw',
+);
+
+
+has close_reason => (
+  is => 'rw',
+);
+
+
+has close_error => (
+  is => 'rw',
+);
+
+foreach my $type (qw( each_message next_message finish parse_error ))
 {
   has "_${type}_cb" => (
     is       => 'ro',
@@ -63,6 +85,7 @@ sub BUILD
   my $are_callbacks_supposed_to_be_ready = 0;
   
   my $finish = sub {
+    my(undef, undef, $message) = @_;
     my $strong_self = $self; # preserve $self because otherwise $self can be destroyed in the callbacks.
     return if $self->_is_finished;
     eval
@@ -74,12 +97,13 @@ sub BUILD
     $self->handle->push_shutdown;
     $self->_is_read_open(0);
     $self->_is_write_open(0);
-    $_->($self) for @{ $self->_finish_cb };
+    $self->close_error($message) if defined $message;
+    $_->($self, $message) for @{ $self->_finish_cb };
   };
   $self->handle->on_error($finish);
   $self->handle->on_eof($finish);
 
-  my $frame = Protocol::WebSocket::Frame->new;
+  my $frame = Protocol::WebSocket::Frame->new( maybe max_payload_size => $self->max_payload_size );
 
   my $read_cb = sub {
     my ($handle) = @_;
@@ -103,12 +127,13 @@ sub BUILD
         {
           push(@temp_messages, $message);
         }
-      };
+      }
       1; # succeed to parse.
     };
     if(!$success)
     {
       $self->_force_shutdown();
+      $_->($self, $@) for @{ $self->_parse_error_cb };
     }
   };
 
@@ -164,6 +189,13 @@ sub _process_message
   }
   elsif($received_message->is_close)
   {
+    my $body = $received_message->body;
+    if($body)
+    {
+      my($code, $reason) = unpack 'na*', $body;
+      $self->close_code($code);
+      $self->close_reason(Encode::decode('UTF-8', $reason));
+    }
     $self->_is_read_open(0);
     $self->close();
   }
@@ -191,12 +223,12 @@ sub send
   
   if(ref $message)
   {
-    $frame = Protocol::WebSocket::Frame->new(buffer => $message->body, masked => $self->masked);
+    $frame = Protocol::WebSocket::Frame->new(buffer => $message->body, masked => $self->masked, max_payload_size => 0);
     $frame->opcode($message->opcode);
   }
   else
   {
-    $frame = Protocol::WebSocket::Frame->new(buffer => $message, masked => $self->masked);
+    $frame = Protocol::WebSocket::Frame->new(buffer => $message, masked => $self->masked, max_payload_size => 0);
   }
   $self->handle->push_write($frame->to_bytes);
   $self;
@@ -219,6 +251,10 @@ sub on
   {
     push @{ $self->_finish_cb }, $cb;
   }
+  elsif($event eq 'parse_error')
+  {
+    push @{ $self->_parse_error_cb }, $cb;
+  }
   else
   {
     Carp::croak "unrecongized event: $event";
@@ -258,7 +294,7 @@ AnyEvent::WebSocket::Connection - WebSocket connection for AnyEvent
 
 =head1 VERSION
 
-version 0.35_02
+version 0.40_01
 
 =head1 SYNOPSIS
 
@@ -321,6 +357,25 @@ If set to true, it masks outgoing frames. The default is false.
 The subprotocol returned by the server.  If no subprotocol was requested, it
 may be C<undef>.
 
+=head2 max_payload_size
+
+The maximum payload size for received frames.  Currently defaults to whatever
+L<Protocol::WebSocket> defaults to.
+
+=head2 close_code
+
+If provided by the other side, the code that was provided when the 
+connection was closed.
+
+=head2 close_reason
+
+If provided by the other side, the reason for closing the connection.
+
+=head2 close_error
+
+If the connection is closed due to a network error, this will hold the
+message.
+
 =head1 METHODS
 
 =head2 send
@@ -354,11 +409,22 @@ Called each time a message is received from the WebSocket.
 
 Called only for the next message received from the WebSocket.
 
+=head3 parse_error
+
+ $cb->($connection, $text_error_message)
+
+Called if there is an error parsing a message sent from the remote end.
+After this callback is called, the connection will be closed.
+Among other possible errors, this event will trigger if a frame has a
+payload which is larger that C<max_payload_size>.
+
 =head3 finish
 
- $cb->($connection)
+ $cb->($connection, $message)
 
-Called when the connection is terminated
+Called when the connection is terminated.  If the connection is terminted
+due to an error, the message will be provided as the second argument.
+On a cleanly closed connection this will be `undef`.
 
 =head2 close
 
@@ -419,6 +485,8 @@ Toshio Ito
 
 José Joaquín Atria
 
+Kivanc Yazan
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2013 by Graham Ollis.
