@@ -6,7 +6,8 @@ use Moo;
 use Encode ();
 
 # ABSTRACT: WebSocket message for AnyEvent
-our $VERSION = '0.35_02'; # TRIAL VERSION
+our $VERSION = '0.40_01'; # TRIAL VERSION
+$VERSION = eval $VERSION;
 
 
 has body => ( is => 'ro', required => 1 );
@@ -39,7 +40,7 @@ AnyEvent::WebSocket::Message - WebSocket message for AnyEvent
 
 =head1 VERSION
 
-version 0.35_02
+version 0.40_01
 
 =head1 SYNOPSIS
 
@@ -57,8 +58,10 @@ version 0.35_02
 
 =head1 DESCRIPTION
 
-Instances of this class represent a message passed through the WebSocket
-from the other end.
+Instances of this class represent a single WebSocket message.  They are
+the objects that come through from the other end of your
+L<AnyEvent::WebSocket::Connection> instance.  They can also be sent through
+that class using its C<send> method.
 
 =head1 ATTRIBUTES
 
@@ -146,6 +149,8 @@ Toshio Ito
 
 José Joaquín Atria
 
+Kivanc Yazan
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2013 by Graham Ollis.
